import nbformat
import re
from pathlib import Path

_pattern = r"^#\s*export\s*$"
_flags = re.IGNORECASE | re.MULTILINE


def export(notebook_path: str, output_path: str, delete_export_comments: bool = False):
    """
    Export cells to Python script.

    Parameters
    ----------
    notebook_path: str
                    Path of the notebook that contains cells to be exported.
    output_path: str
                    Path of the Python script cells will be exported. Note that if the script is existed, export()
                    will override it. If not existed, it will create the file.
    delete_export_comments: bool
                    Whether keep the '#export' or '# export' comments in the output file.

    Returns
    ------
    None
    """

    notebook_path = Path(notebook_path)
    output_path = Path(output_path)
    # Check whether the notebook exists or not.
    if not notebook_path.exists():
        raise FileNotFoundError(notebook_path)
    # Read notebook
    with open(notebook_path, 'r', encoding='utf-8') as inp:
        notebook = nbformat.reads(inp.read(), as_version=4)
    # Iterate over cells
    sources = []
    for cell in notebook['cells']:
        #  Find cells that will be exported
        if cell['cell_type'] == 'code' and re.search(_pattern, cell['source'], _flags) is not None:
            if delete_export_comments:
                source = re.sub(_pattern, '', cell['source'], flags=_flags)
            else:
                source = cell['source']
            sources.append(source)
    if len(sources) > 0:
        #  Write source codes to output file
        with open(output_path, 'w', encoding='UTF-8') as out:
            out.write(f"# THIS SCRIPT IS AUTOGENERATED FROM {notebook_path}. \n\n")
            for index, source in enumerate(sources):
                if not delete_export_comments:
                    out.write("\n")
                out.write(source)
                if index == len(sources) - 1:
                    # Put single new line at the end of script.
                    out.write("\n")
                else:
                    # Put double new line between different cells.
                    out.write("\n\n")
    else:
        print("No cell exported.")



import sys
import argparse
from .exporter import export

# Prepare command line interface
parser = argparse.ArgumentParser(description='Export Jupyter Notebooks as Python scripts.')
parser.add_argument('notebook_path', type=str, help="Path of the Jupyter notebook.")
parser.add_argument('output_path', type=str, help="Path of the output script.")
parser.add_argument('--delete_export_comments', '-dc', action='store_true',
                    help="If exists, then #export comments will be deleted from output script.")


def run_export_console_script():
    """
    Run export() function from command line.
    """

    #  Parse command line arguments
    args = parser.parse_args(args=sys.argv[1:])
    try:
        export(args.notebook_path, args.output_path, args.delete_export_comments)
    except FileNotFoundError as e:
        print(f"Notebook file not found. {e} not exists.")


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name="exporter",
    version="0.0.2",
    author="Hasan Yaman",
    author_email="hasannyaman@gmail.com",
    description="exporter helps to export Jupyter notebooks as a Python script.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hasan-yaman/Exporter",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.6",
    install_requires=[
        'nbformat'
    ],
    entry_points={
        'console_scripts': [
            'exporter=exporter.console_script:run_export_console_script'
        ]
    }
)
